local ADDON, Package = ...

local Color = {}

Color.CYAN = CreateColor(0x00 / 255.0, 0xFF / 255.0, 0xFF / 255.0)
Color.LA_PALMA = CreateColor(0x19 / 255.0, 0x99 / 255.0, 0x19 / 255.0)
Color.JORDY_BLUE = CreateColor(0x91 / 255.0, 0xD7 / 255.0, 0xF2 / 255.0)
Color.SPRAY = CreateColor(0x72 / 255.0, 0xF2 / 255.0, 0xE5 / 255.0)
Color.YELLOW = CreateColor(1, 1, 0)

function Color.rgb_float_from_hex(hex)
  local r = bit.band(bit.rshift(hex, 16), 0xFF)
  local g = bit.band(bit.rshift(hex, 8), 0xFF)
  local b = bit.band(hex, 0xFF)

  return r / 255.0, g / 255.0, b / 255.0
end

function Color.rgb_float_from_hex_string(hex_string)
  return Color.rgb_float_from_hex(tonumber(hex_string, 16))
end

function Color.rgb_hex_from_float(r, g, b)
  return format("%02x%02x%02x", r * 255, g * 255, b * 255)
end

function Color.rgb_to_hsl(r, g, b)
  local min, max = math.min(r, g, b), math.max(r, g, b)
  local h, s, l = 0, 0, (max + min) / 2

  if min == max then
    h, s = 0, 0
  else
    local delta = max - min

    s = l > 0.5 and delta / (2 - max - min) or delta / (max + min)

    if max == r then
      h = (g - b) / delta + (g < b and 6 or 0)
    elseif max == g then
      h = (b - r) / delta + 2
    elseif max == b then
      h = (r - g) / delta + 4
    end

    h = h / 6
  end

  return h, s, l
end

function Color.hue_to_rgb(p, q, t)
  if t < 0 then
    t = t + 1
  end

  if t > 1 then
    t = t - 1
  end

  if t < 1 / 6 then
    return p + (q - p) * 6 * t
  end

  if t < 1 / 2 then
    return q
  end

  if t < 2 / 3 then
    return p + (q - p) * (2 / 3 - t) * 6
  end

  return p
end

function Color.hsl_to_rgb(h, s, l)
  local r, g, b = 0, 0, 0

  if s == 0 then
    r, g, b = l, l, l
  else
    local q = l < 0.5 and l * (1 + s) or l + s - l * s
    local p = 2 * l - q

    r = Color.hue_to_rgb(p, q, h + 1 / 3)
    g = Color.hue_to_rgb(p, q, h)
    b = Color.hue_to_rgb(p, q, h - 1 / 3)
  end

  return r, g, b
end

function Color.darken(r, g, b, amount)
  local h, s, l = Color.rgb_to_hsl(r, g, b)

  l = l - (amount / 100)
  l = math.min(1, math.max(0, l))

  return Color.hsl_to_rgb(h, s, l)
end

function Color.darken_hex_markup(hex_markup, amount)
  local r, g, b = Color.rgb_float_from_hex_string(hex_markup:lower():gsub("|cff", ""))

  return "|cff" .. Color.rgb_hex_from_float(Color.darken(r, g, b, amount))
end

Package.Color = Color
