local ADDON, Package = ...

local Color = Package.Color

local Module = {}

local CHAT_FRAMES = {
  ChatFrame1 = ChatFrame1,
  ChatFrame2 = ChatFrame2,
  ChatFrame3 = ChatFrame3,
  ChatFrame4 = ChatFrame4,
  ChatFrame5 = ChatFrame5,
  ChatFrame6 = ChatFrame6,
  ChatFrame7 = ChatFrame7,
  ChatFrame8 = ChatFrame8,
  ChatFrame9 = ChatFrame9,
  ChatFrame10 = ChatFrame10
}

local URL_DELIMITER = "://"

local URL_MATCH = "^(.*)(https?://[^ ]+)(.*)$"

function Module.configure_alt_arrow_key_mode()
  for name, frame in pairs(CHAT_FRAMES) do
    if name ~= "ChatFrame2" then
      frame.editBox:SetAltArrowKeyMode(false)
    end
  end
end

function Module.configure_max_lines()
  for name, frame in pairs(CHAT_FRAMES) do
    if name ~= "ChatFrame2" then
      frame:SetMaxLines(4096)
    end
  end
end

function Module.configure_clamp_rect_insets()
  for _, frame in pairs(CHAT_FRAMES) do
    frame:SetClampRectInsets(0, 0, 0, 0)
  end

  hooksecurefunc("FloatingChatFrame_UpdateBackgroundAnchors", function (self)
    self:SetClampRectInsets(0, 0, 0, 0)
  end)
end

function Module.configure_raid_warning()
  ChatTypeInfo.RAID_WARNING.sticky = 1
end

function Module.configure_logger()
  local darken_hex_markup = function (hex_markup)
    return Color.darken_hex_markup(hex_markup, 10)
  end

  local darken_message = function(message, r, g, b, ...)
    if message then
      r, g, b = Color.darken(r or 1, g or 1, b or 1, 20)
      message = message:gsub("(|cff%x%x%x%x%x%x)", darken_hex_markup)
    end

    return message, r, g, b, ...
  end

  for name, messages in pairs(MedusaPreludeStoragePerCharacter.chat_log) do
    for at = #messages, 1, -1 do
      CHAT_FRAMES[name]:AddMessage(darken_message(unpack(messages[at])))
    end
  end

  local f = function (self, message, ...)
    if message and message:find("BNplayer") then
      local account_id = message:match(" |HBNplayer:[^:]+:([0-9]+).+|h ")
      local _, _, battle_tag = BNGetFriendInfoByID(account_id or 0)

      if battle_tag then
        message = message:gsub("^([^|]+)|HBNplayer:[^%[]+%[[^%]]+%]|h", "%1[" .. battle_tag:gsub("#[0-9]+", "") .. "]")
      end
    end

    local storage_per_character = MedusaPreludeStoragePerCharacter.chat_log[self:GetName()]

    table.insert(storage_per_character, 1, {message, ...})

    if #storage_per_character > 64 then
      table.remove(storage_per_character)
    end
  end

  for name, frame in pairs(CHAT_FRAMES) do
    if name ~= "ChatFrame2" then
      hooksecurefunc(frame, "AddMessage", f)
    end
  end
end

function Module.configure_to_copy_text_to_clipboard()
  local function on_click(self)
    local is_flagged = self:GetChecked()

    self:SetButtonState("PUSHED", is_flagged)

    for at = 1, 10 do
      local frame = _G["ChatFrame" .. at]
      frame:EnableMouse(is_flagged)
      frame:SetTextCopyable(is_flagged)
    end
  end

  local frame = CreateFrame("CheckButton", ADDON .. "ChatClipboardButton", ChatFrame1)
  frame:SetSize(16, 16)
  frame:SetPoint("TOPRIGHT")
  frame:SetNormalTexture("Interface\\AddOns\\MedusaPrelude\\media\\clipboard-solid.tga")
  frame:SetPushedTexture("Interface\\AddOns\\MedusaPrelude\\media\\cut-solid.tga")
  frame:SetScript("OnClick", on_click)

  local background = frame:CreateTexture(nil, "BACKGROUND")
  background:SetAllPoints(frame)
  background:SetTexture("Interface\\Buttons\\WHITE8X8")
  background:SetVertexColor(0.5, 0.5, 0.5, 0.5)
end

function Module.configure_to_copy_url_to_clipboard()
  local f = function (self, link, text, button)
    if link:sub(1, 4) == "item" and text:find(URL_DELIMITER) then
      StaticPopup_Show(ADDON .. "Clipboard", nil, nil, text:match("%[(.+)%]"))
    end
  end

  hooksecurefunc("ChatFrame_OnHyperlinkShow", f)

  local events = {
    "SYSTEM", "SAY", "PARTY", "RAID", "GUILD", "OFFICER", "YELL", "WHISPER", "WHISPER_INFORM", "REPLY",
    "EMOTE", "TEXT_EMOTE", "CHANNEL", "CHANNEL_NOTICE", "CHANNEL_NOTICE_USER", "AFK", "DND",
    "RAID_LEADER", "RAID_WARNING", "INSTANCE_CHAT", "INSTANCE_CHAT_LEADER", "PARTY_LEADER",
    "BN_WHISPER", "BN_WHISPER_INFORM", "BN_WHISPER_PLAYER_OFFLINE"
  }

  local message_event_filter = function (self, event, message, ...)
    if message and message:find(URL_DELIMITER) then
      local pre, url, post = message:match(URL_MATCH)

      if pre and url and post then
        message = pre .. string.format("|H%s:|h[%s]|h", "item", url) .. post
      end
    end

    return false, message, ...
  end

  for _, event in pairs(events) do
    ChatFrame_AddMessageEventFilter("CHAT_MSG_" .. event, message_event_filter)
  end
end

StaticPopupDialogs[ADDON .. "Clipboard"] = {
  button1 = CLOSE,
  whileDead = true,
  hideOnEscape = true,
  hasEditBox = true,
  timeout = 0,
  text = "",
  OnShow = function (self)
    self.editBox:SetText(self.data)
    self.editBox:HighlightText()
    self.editBox:SetCursorPosition(0)

    if not self.editBox:GetScript("OnMouseUp") then
      self.editBox:SetScript("OnMouseUp", function (self) self:HighlightText() end)
    end
  end,
  OnHide = function (self)
    self.data = nil
  end,
  EditBoxOnEscapePressed = function(self)
    self:GetParent():Hide()

    ClearCursor()
  end,
  EditBoxOnTextChanged = function (self)
    self:SetText(self:GetParent().data)
    self:HighlightText()
  end
}

Package.Chat = Module
